@@ -1,13 +1,16 @@
 #' Functions that extract values from the fitted model
 #'
-#' These functions allow extracting orders and lags for \code{ssarima()}, \code{ges()} and \code{sma()},
-#' type of model from \code{es()} and \code{ces()} and error type of all the above.
+#' These functions allow extracting orders and lags for \code{ssarima()}, \code{gum()} and \code{sma()},
+#' type of model from \code{es()} and \code{ces()} and name of model.
 #'
-#' \code{orders()} and \code{lags()} are useful only for SSARIMA, GES and SMA. They return \code{NA} for other functions.
+#' \code{orders()} and \code{lags()} are useful only for SSARIMA, GUM and SMA. They return \code{NA} for other functions.
 #' This can also be applied to \code{arima()}, \code{Arima()} and \code{auto.arima()} functions from stats and forecast packages.
 #' \code{modelType()} is useful only for ETS and CES. They return \code{NA} for other functions.
-#' This can also be applied to \code{ets()} function from forecast package. Finally, \code{errorType}
-#' extracts the type of error from the model (either additive or multiplicative).
+#' This can also be applied to \code{ets()} function from forecast package. \code{errorType}
+#' extracts the type of error from the model (either additive or multiplicative). Finally, \code{modelName}
+#' returns the name of the fitted model. For example, "ARIMA(0,1,1)". This is purely descriptive and
+#' can also be applied to non-smooth classes, so that, for example, you can easily extract the name
+#' of the fitted AR model from \code{ar()} function from \code{stats} package.
 #'
 #' @template ssAuthor
 #' @template ssKeywords
@@ -15,13 +18,15 @@
 #' @aliases orders
 #' @param object Model estimated using one of the functions of smooth package.
 #' @param ... Currently nothing is accepted via ellipsis.
-#' @return     Either vector or list with values is returned.
+#' @return     Either vector, scalar or list with values is returned.
 #' \code{orders()} in case of ssarima returns list of values:
 #' \itemize{
 #' \item \code{ar} - AR orders.
 #' \item \code{i} - I orders.
 #' \item \code{ma} - MA orders.
 #' }
+#' \code{lags()} returns the vector of lags of the model.
+#' All the other functions return strings of character.
 #' @seealso \link[forecast]{forecast}, \link[smooth]{ssarima}
 #' @examples
 #'
@@ -32,19 +37,24 @@
 #' orders(ourModel)
 #' lags(ourModel)
 #' modelType(ourModel)
-#' errorType(ourModel)
-#' # And as another example it does the opposite for ges() and ssarima()
-#' ourModel <- ges(x, h=10, orders=c(1,1), lags=c(1,4))
+#' modelName(ourModel)
+#'
+#' # And as another example it does the opposite for gum() and ssarima()
+#' ourModel <- gum(x, h=10, orders=c(1,1), lags=c(1,4))
 #' orders(ourModel)
 #' lags(ourModel)
 #' modelType(ourModel)
-#' errorType(ourModel)
+#' modelName(ourModel)
 #'
 #' # Finally these values can be used for simulate functions or original functions.
 #' ourModel <- auto.ssarima(x)
 #' ssarima(x, orders=orders(ourModel), lags=lags(ourModel), constant=ourModel$constant)
 #' sim.ssarima(orders=orders(ourModel), lags=lags(ourModel), constant=ourModel$constant)
 #'
+#' ourModel <- es(x)
+#' es(x, model=modelType(ourModel))
+#' sim.es(model=modelType(ourModel))
+#'
 #' @rdname orders
 #' @export orders
 orders <- function(object, ...) UseMethod("orders")
@@ -54,6 +64,11 @@ orders <- function(object, ...) UseMethod("orders")
 #' @export lags
 lags <- function(object, ...) UseMethod("lags")
 
+#' @aliases modelName
+#' @rdname orders
+#' @export modelName
+modelName <- function(object, ...) UseMethod("modelName")
+
 #' @aliases modelType
 #' @rdname orders
 #' @export modelType
@@ -61,10 +76,7 @@ modelType <- function(object, ...) UseMethod("modelType")
 
 modelLags <- function(object, ...) UseMethod("modelLags")
 
-#' @aliases errorType
-#' @rdname orders
-#' @export errorType
-errorType <- function(object, ...) UseMethod("errorType")
+smoothType <- function(object, ...) UseMethod("smoothType")
 
 ##### Likelihood function and stuff #####
 
@@ -112,7 +124,7 @@ BICc.smooth <- function(object, ...){
 #' Function returns the covariance matrix of conditional multiple steps ahead forecast errors
 #'
 #' This function extracts covariance matrix of 1 to h steps ahead forecast errors for
-#' \code{ssarima()}, \code{ges()}, \code{sma()}, \code{es()} and \code{ces()} models.
+#' \code{ssarima()}, \code{gum()}, \code{sma()}, \code{es()} and \code{ces()} models.
 #'
 #' The function returns either scalar (if it is a non-smooth model)
 #' or the matrix of (h x h) size with variances and covariances of 1 to h steps ahead
@@ -209,6 +221,7 @@ covar.smooth <- function(object, type=c("analytical","empirical","simulated"), .
     }
     # Simulated covariance matrix
     else if(type=="s"){
+        smoothType <- smoothType(object);
         ellipsis <- list(...);
         if(any(names(ellipsis)=="obs")){
             obs <- ellipsis$obs;
@@ -230,23 +243,23 @@ covar.smooth <- function(object, type=c("analytical","empirical","simulated"), .
         }
 
         h <- length(object$forecast);
-        if(gregexpr("ETS",object$model)!=-1){
+        if(smoothType=="ETS"){
             smoothFunction <- es;
         }
-        # GES models
-        else if(gregexpr("GES",object$model)!=-1){
-            smoothFunction <- ges;
+        # GUM models
+        else if(smoothType=="GUM"){
+            smoothFunction <- gum;
         }
         # SSARIMA models
-        else if(gregexpr("ARIMA",object$model)!=-1){
+        else if(smoothType=="ARIMA"){
             smoothFunction <- ssarima;
         }
         # CES models
-        else if(gregexpr("CES",object$model)!=-1){
+        else if(smoothType=="CES"){
             smoothFunction <- ces;
         }
         # SMA models
-        else if(gregexpr("SMA",object$model)!=-1){
+        else if(smoothType=="SMA"){
             smoothFunction <- sma;
         }
 
@@ -257,7 +270,7 @@ covar.smooth <- function(object, type=c("analytical","empirical","simulated"), .
             # Apply the model to the simulated data
             smoothModel <- smoothFunction(newData$data[,i], model=object, h=h);
             # Remove first h-1 and last values.
-            errors <- smoothModel$errors[h-1+1:obs,];
+            errors <- smoothModel$errors;
 
             # Transform errors if needed
             if(errorType(object)=="M"){
@@ -294,7 +307,7 @@ covar.smooth <- function(object, type=c("analytical","empirical","simulated"), .
         transition <- object$transition;
         measurement <- object$measurement;
 
-        covarMat <- covarAnal(lagsModel, h, nComponents, measurement, transition, persistence, s2);
+        covarMat <- covarAnal(lagsModel, h, measurement, transition, persistence, s2);
 
     }
     return(covarMat);
@@ -309,8 +322,7 @@ logLik.smooth <- function(object,...){
         return(NULL);
     }
     else{
-        obs <- nobs(object);
-        return(structure(object$logLik,nobs=obs,df=nParam(object),class="logLik"));
+        return(structure(object$logLik,nobs=nobs(object),df=nParam(object),class="logLik"));
     }
 }
 #' @export
@@ -481,9 +493,15 @@ pls.smooth <- function(object, holdout=NULL, ...){
             plsValue <- sum(ds(errors, 0, b, log=TRUE));
         }
         else{
-        # Here and later in the code the abs() is needed for weird cases of wrong covarMat
-            plsValue <- -as.vector(log(2*pi*abs(det(covarMat)))/2 +
-                                       (t(errors) %*% solve(covarMat) %*% errors) / 2);
+            if(is.infinite(det(covarMat))){
+                plsValue <- -as.vector((log(2*pi)+(abs(determinant(covarMat)$modulus)))/2 +
+                                           (t(errors) %*% solve(covarMat) %*% errors) / 2);
+            }
+            else{
+                # Here and later in the code the abs() is needed for weird cases of wrong covarMat
+                plsValue <- -as.vector(log(2*pi*abs(det(covarMat)))/2 +
+                                           (t(errors) %*% solve(covarMat) %*% errors) / 2);
+            }
         }
         return(plsValue);
     }
@@ -545,63 +563,19 @@ pls.smooth <- function(object, holdout=NULL, ...){
     return(plsValue);
 }
 
-
-#' Point likelihood values
-#'
-#' This function returns a vector of logarithms of likelihoods for each observation
-#'
-#' Instead of taking the expected log-likelihood for the whole series, this function
-#' calculates the individual value for each separate observation. Note that these
-#' values are biased, so you would possibly need to take number of degrees of freedom
-#' into account in order to have an unbiased estimator.
-#'
-#' This value is based on the general likelihood (not its concentrated version), so
-#' the sum of these values may slightly differ from the output of logLik.
-#'
-#' @aliases pointLik
-#' @param object Time series model.
-#' @param ...  Some stuff.
-#' @return This function returns a vector.
-#' @template ssAuthor
-#' @seealso \link[stats]{AIC}, \link[stats]{BIC}
-#' @keywords htest
-#' @examples
-#'
-#' ourModel <- ces(rnorm(100,0,1),h=10)
-#'
-#' pointLik(ourModel)
-#'
-#' # Bias correction
-#' pointLik(ourModel) - nParam(ourModel)
-#'
-#' # Bias correction in AIC style
-#' 2*(nParam(ourModel)/nobs(ourModel) - pointLik(ourModel))
-#'
-#' # BIC calculation based on pointLik
-#' log(nobs(ourModel))*nParam(ourModel) - 2*sum(pointLik(ourModel))
-#'
-#' @export pointLik
-pointLik <- function(object, ...) UseMethod("pointLik")
-
+#' @importFrom stats sigma
 #' @export
-pointLik.default <- function(object, ...){
-    obs <- nobs(object);
-    errors <- residuals(object);
-    likValues <- dnorm(errors, 0, sigma(object), TRUE);
-
-    return(likValues);
+sigma.smooth <- function(object, ...){
+    return(sqrt(object$s2));
 }
 
 #' @export
-pointLik.ets <- function(object, ...){
-    likValues <- pointLik.default(object);
-    if(errorType(object)=="M"){
-        likValues <- likValues - log(abs(fitted(object)));
-    }
-
-    return(likValues);
+sigma.ets <- function(object, ...){
+    return(sqrt(object$sigma2));
 }
 
+#### pointLik for smooth ####
+#' @importFrom greybox pointLik
 #' @export
 pointLik.smooth <- function(object, ...){
     obs <- nobs(object);
@@ -629,62 +603,16 @@ pointLik.smooth <- function(object, ...){
     return(likValues);
 }
 
-#' Point AIC
-#'
-#' This function returns a vector of AIC values for the in-sample observations
-#'
-#' This is based on \link[smooth]{pointLik} function. The formula for this is:
-#' pAIC_t = 2 * k - 2 * T * l_t ,
-#' where k is the number of parameters, T is the number of observations and l_t is
-#' the point likelihood. This way we preserve the property that AIC = mean(pAIC).
-#'
-#' @aliases pAIC
-#' @param object Time series model.
-#' @param ...  Some stuff.
-#' @return The function returns the vector of point AIC values.
-#' @template ssAuthor
-#' @seealso \link[smooth]{pointLik}
-#' @keywords htest
-#' @examples
-#'
-#' ourModel <- ces(rnorm(100,0,1),h=10)
-#'
-#' pAICValues <- pAIC(ourModel)
-#'
-#' mean(pAICValues)
-#' AIC(ourModel)
-#'
-#' @export pAIC
-pAIC <- function(object, ...) UseMethod("pAIC")
-
-#' @export
-pAIC.default <- function(object, ...){
-    obs <- nobs(object);
-    k <- nParam(object);
-    return(2*k - 2 * obs * pointLik(object));
-}
-
-
-#' @importFrom stats sigma
-#' @export
-sigma.smooth <- function(object, ...){
-    return(sqrt(object$s2));
-}
-
-#' @export
-sigma.ets <- function(object, ...){
-    return(sqrt(object$sigma2));
-}
-
 #### Extraction of parameters of models ####
 #' @export
 coef.smooth <- function(object, ...)
 {
-    if(gregexpr("CES",object$model)!=-1){
+    smoothType <- smoothType(object);
+    if(smoothType=="CES"){
         parameters <- c(object$A,object$B);
     }
-    else if(gregexpr("ETS",object$model)!=-1){
-        if(any(unlist(gregexpr("C",object$model))==-1)){
+    else if(smoothType=="ETS"){
+        if(any(unlist(gregexpr("C",modelType(object)))==-1)){
             # If this was normal ETS, return values
             parameters <- c(object$persistence,object$initial,object$initialSeason,object$initialX);
         }
@@ -694,24 +622,24 @@ coef.smooth <- function(object, ...)
             parameters <- NULL;
         }
     }
-    else if(gregexpr("GES",object$model)!=-1){
+    else if(smoothType=="GUM"){
         parameters <- c(object$measurement,object$transition,object$persistence,object$initial);
         names(parameters) <- c(paste0("Measurement ",c(1:length(object$measurement))),
                                paste0("Transition ",c(1:length(object$transition))),
                                paste0("Persistence ",c(1:length(object$persistence))),
                                paste0("Initial ",c(1:length(object$initial))));
     }
-    else if(gregexpr("ARIMA",object$model)!=-1){
+    else if(smoothType=="ARIMA"){
         if(any(unlist(gregexpr("combine",object$model))==-1)){
             # If this was normal ARIMA, return values
             namesConstant <- NamesMA <- NamesAR <- parameters <- NULL;
             if(any(object$AR!=0)){
                 parameters <- c(parameters,object$AR);
-                NamesAR <- paste(rownames(object$AR),rep(colnames(object$AR),each=ncol(object$AR)),sep=", ");
+                NamesAR <- paste(rownames(object$AR),rep(colnames(object$AR),each=nrow(object$AR)),sep=", ");
             }
             if(any(object$MA!=0)){
                 parameters <- c(parameters,object$MA);
-                NamesMA <- paste(rownames(object$MA),rep(colnames(object$MA),each=ncol(object$MA)),sep=", ")
+                NamesMA <- paste(rownames(object$MA),rep(colnames(object$MA),each=nrow(object$MA)),sep=", ")
             }
             if(object$constant!=0){
                 parameters <- c(parameters,object$constant);
@@ -726,7 +654,7 @@ coef.smooth <- function(object, ...)
             parameters <- NULL;
         }
     }
-    else if(gregexpr("SMA",object$model)!=-1){
+    else if(smoothType=="SMA"){
         parameters <- object$persistence;
     }
 
@@ -753,7 +681,7 @@ NULL
 #' Hyndman's "forecast" package
 #'
 #' This is not a compulsory function. You can simply use \link[smooth]{es},
-#' \link[smooth]{ces}, \link[smooth]{ges} or \link[smooth]{ssarima} without
+#' \link[smooth]{ces}, \link[smooth]{gum} or \link[smooth]{ssarima} without
 #' \code{forecast.smooth}. But if you are really used to \code{forecast}
 #' function, then go ahead!
 #'
@@ -764,12 +692,12 @@ NULL
 #' details.
 #' @param level Confidence level. Defines width of prediction interval.
 #' @param ...  Other arguments accepted by either \link[smooth]{es},
-#' \link[smooth]{ces}, \link[smooth]{ges} or \link[smooth]{ssarima}.
+#' \link[smooth]{ces}, \link[smooth]{gum} or \link[smooth]{ssarima}.
 #' @return Returns object of class "smooth.forecast", which contains:
 #'
 #' \itemize{
-#' \item \code{model} - the estimated model (ES / CES / GES / SSARIMA).
-#' \item \code{method} - the name of the estimated model (ES / CES / GES / SSARIMA).
+#' \item \code{model} - the estimated model (ES / CES / GUM / SSARIMA).
+#' \item \code{method} - the name of the estimated model (ES / CES / GUM / SSARIMA).
 #' \item \code{fitted} - fitted values of the model.
 #' \item \code{actuals} - actuals provided in the call of the model.
 #' \item \code{forecast} aka \code{mean} - point forecasts of the model
@@ -799,30 +727,36 @@ NULL
 forecast.smooth <- function(object, h=10,
                             intervals=c("parametric","semiparametric","nonparametric","none"),
                             level=0.95, ...){
+    smoothType <- smoothType(object);
     intervals <- intervals[1];
-    if(gregexpr("ETS",object$model)!=-1){
+    if(smoothType=="ETS"){
         newModel <- es(object$actuals,model=object,h=h,intervals=intervals,level=level,silent="all",...);
     }
-    else if(gregexpr("CES",object$model)!=-1){
+    else if(smoothType=="CES"){
         newModel <- ces(object$actuals,model=object,h=h,intervals=intervals,level=level,silent="all",...);
     }
-    else if(gregexpr("GES",object$model)!=-1){
-        newModel <- ges(object$actuals,model=object,h=h,intervals=intervals,level=level,silent="all",...);
+    else if(smoothType=="GUM"){
+        newModel <- gum(object$actuals,model=object,type=errorType(object),h=h,intervals=intervals,level=level,silent="all",...);
     }
-    else if(gregexpr("ARIMA",object$model)!=-1){
+    else if(smoothType=="ARIMA"){
         if(any(unlist(gregexpr("combine",object$model))==-1)){
-            newModel <- ssarima(object$actuals,model=object,h=h,intervals=intervals,level=level,silent="all",...);
+            if(any(class(object)=="msarima")){
+                newModel <- msarima(object$actuals,model=object,h=h,intervals=intervals,level=level,silent="all",...);
+            }
+            else{
+                newModel <- ssarima(object$actuals,model=object,h=h,intervals=intervals,level=level,silent="all",...);
+            }
         }
         else{
             stop(paste0("Sorry, but in order to produce forecasts for this ARIMA we need to recombine it.\n",
                  "You will have to use auto.ssarima() function instead."),call.=FALSE);
         }
     }
-    else if(gregexpr("SMA",object$model)!=-1){
+    else if(smoothType=="SMA"){
         newModel <- sma(object$actuals,model=object,h=h,intervals=intervals,level=level,silent="all",...);
     }
     else{
-        stop("Wrong object provided. This needs to be either 'ETS' or 'CES' or 'GES' or 'SSARIMA' model.",call.=FALSE);
+        stop("Wrong object provided. This needs to be either 'ETS', or 'CES', or 'GUM', or 'SSARIMA', or 'SMA' model.",call.=FALSE);
     }
     output <- list(model=object,method=object$model,fitted=newModel$fitted,actuals=newModel$actuals,
                    forecast=newModel$forecast,lower=newModel$lower,upper=newModel$upper,level=newModel$level,
@@ -844,27 +778,38 @@ getResponse.smooth.forecast <- function(object, ...){
 #### Function extracts lags of provided model ####
 #' @export
 lags.default <- function(object, ...){
-    lags <- NA;
+    return(NA);
+}
+
+#' @export
+lags.ets <- function(object, ...){
+    modelName <- modelType(object);
+    lags <- c(1);
+    if(substr(modelName,nchar(modelName),nchar(modelName))!="N"){
+        lags <- c(lags,frequency(getResponse(object)));
+    }
     return(lags);
 }
 
+#' @export
+lags.ar <- function(object, ...){
+    return(1);
+}
+
 #' @export
 lags.Arima <- function(object, ...){
-    model <- object$arma;
-
-    lags <- c(1,model[5]);
-
-    return(lags);
+    return(c(1,object$arma[5]));
 }
 
 #' @export
 lags.smooth <- function(object, ...){
     model <- object$model;
+    smoothType <- smoothType(object);
     if(!is.null(model)){
-        if(gregexpr("GES",model)!=-1){
+        if(smoothType=="GUM"){
             lags <- as.numeric(substring(model,unlist(gregexpr("\\[",model))+1,unlist(gregexpr("\\]",model))-1));
         }
-        else if(gregexpr("ARIMA",model)!=-1){
+        else if(smoothType=="ARIMA"){
             if(any(unlist(gregexpr("combine",object$model))==-1)){
                 if(any(unlist(gregexpr("\\[",model))!=-1)){
                     lags <- as.numeric(substring(model,unlist(gregexpr("\\[",model))+1,unlist(gregexpr("\\]",model))-1));
@@ -877,17 +822,17 @@ lags.smooth <- function(object, ...){
                 warning("ARIMA was combined and we cannot extract lags anymore. Sorry!",call.=FALSE);
             }
         }
-        else if(gregexpr("SMA",model)!=-1){
+        else if(smoothType=="SMA"){
             lags <- 1;
         }
-        else if(gregexpr("ETS",model)!=-1){
+        else if(smoothType=="ETS"){
             modelName <- modelType(object);
             lags <- c(1);
             if(substr(modelName,nchar(modelName),nchar(modelName))!="N"){
                 lags <- c(lags,frequency(getResponse(object)));
             }
         }
-        else if(gregexpr("CES",model)!=-1){
+        else if(smoothType=="CES"){
             modelName <- modelType(object);
             dataFreq <- frequency(getResponse(object));
             if(modelName=="none"){
@@ -922,25 +867,12 @@ lags.smooth <- function(object, ...){
 lags.smooth.sim <- lags.smooth;
 
 #### Function extracts type of error in the model: "A" or "M" ####
-#' @export
-errorType.default <- function(object, ...){
-    return("A");
-}
-
-#' @export
-errorType.ets <- function(object, ...){
-    if(substr(object$method,5,5)=="M"){
-        return("M");
-    }
-    else{
-        return("A");
-    }
-}
-
+#' @importFrom greybox errorType
 #' @export
 errorType.smooth <- function(object, ...){
+    smoothType <- smoothType(object);
     # ETS models
-    if(gregexpr("ETS",object$model)!=-1){
+    if(smoothType=="ETS"){
         if(any(substr(modelType(object),1,1)==c("A","X"))){
             Etype <- "A";
         }
@@ -948,13 +880,13 @@ errorType.smooth <- function(object, ...){
             Etype <- "M";
         }
         else{
-            stop("Sorry, but we cannot calculate PLS for this type of model",
+            stop("Sorry, but we cannot define error type for this type of model",
                  call.=FALSE);
         }
     }
-    # GES models
-    else if(gregexpr("GES",object$model)!=-1){
-        if(gregexpr("MGES",object$model)!=-1){
+    # GUM models
+    else if(smoothType=="GUM"){
+        if(substr(modelName(object),1,1)=="M"){
             Etype <- "M";
         }
         else{
@@ -962,15 +894,19 @@ errorType.smooth <- function(object, ...){
         }
     }
     # SSARIMA models
-    else if(gregexpr("ARIMA",object$model)!=-1){
+    else if(smoothType=="ARIMA"){
         Etype <- "A";
     }
     # CES models
-    else if(gregexpr("CES",object$model)!=-1){
+    else if(smoothType=="CES"){
         Etype <- "A";
     }
     # SMA models
-    else if(gregexpr("SMA",object$model)!=-1){
+    else if(smoothType=="SMA"){
+        Etype <- "A";
+    }
+    # SMA models
+    else if(smoothType=="CMA"){
         Etype <- "A";
     }
     else{
@@ -991,30 +927,79 @@ errorType.iss <- function(object, ...){
 ##### Function returns the modellags from the model - internal function #####
 modelLags.default <- function(object, ...){
     modelLags <- NA;
-    if(gregexpr("ETS",object$model)!=-1){
-        modelLags <- matrix(rep(lags(object),times=orders(object)),ncol=1);
+    if(any(class(object)=="msarima")){
+        modelLags <- object$modelLags;
     }
-    else if(gregexpr("GES",object$model)!=-1){
-        modelLags <- matrix(rep(lags(object),times=orders(object)),ncol=1);
-    }
-    else if(gregexpr("ARIMA",object$model)!=-1){
-        ordersARIMA <- orders(object);
-        nComponents <- max(ordersARIMA$ar %*% lags(object) + ordersARIMA$i %*% lags(object),
-                           ordersARIMA$ma %*% lags(object));
-        modelLags <- matrix(rep(1,times=nComponents),ncol=1);
-        if(is.numeric(object$constant)){
-            modelLags <- rbind(modellags,1);
+    else{
+        smoothType <- smoothType(object);
+        if(smoothType=="ETS"){
+            modelLags <- matrix(rep(lags(object),times=orders(object)),ncol=1);
+        }
+        else if(smoothType=="GUM"){
+            modelLags <- matrix(rep(lags(object),times=orders(object)),ncol=1);
+        }
+        else if(smoothType=="ARIMA"){
+            ordersARIMA <- orders(object);
+            nComponents <- max(ordersARIMA$ar %*% lags(object) + ordersARIMA$i %*% lags(object),
+                               ordersARIMA$ma %*% lags(object));
+            modelLags <- matrix(rep(1,times=nComponents),ncol=1);
+            if(is.numeric(object$constant)){
+                modelLags <- rbind(modelLags,1);
+            }
+        }
+        else if(smoothType=="CES"){
+            modelLags <- matrix(rep(lags(object),times=orders(object)),ncol=1);
+        }
+        else if(smoothType=="SMA"){
+            modelLags <- matrix(rep(1,times=orders(object)),ncol=1);
         }
     }
-    else if(gregexpr("CES",object$model)!=-1){
-        modelLags <- matrix(rep(lags(object),times=orders(object)),ncol=1);
-    }
-    else if(gregexpr("SMA",object$model)!=-1){
-        modelLags <- matrix(rep(1,times=orders(object)),ncol=1);
-    }
     return(modelLags);
 }
 
+#### Function extracts the full name of a model. For example "ETS(AAN)" ####
+#' @export
+modelName.default <- function(object, ...){
+    return(NA);
+}
+
+#' @export
+modelName.ar <- function(object, ...){
+    return(paste0("AR(",object$order,")"));
+}
+
+#' @export
+modelName.lm <- function(object, ...){
+    return("Regression");
+}
+
+#' @export
+modelName.Arima <- function(object, ...){
+    ordersArma <- object$arma;
+    if(all(ordersArma[c(3,4,7)]==0)){
+        return(paste0("ARIMA(",paste(ordersArma[c(1,6,2)],collapse=","),")"));
+    }
+    else{
+        return(paste0("SARIMA(",paste(ordersArma[c(1,6,2)],collapse=","),")(",
+                      paste(ordersArma[c(3,7,4)],collapse=","),")[",ordersArma[5],"]"));
+    }
+}
+
+#' @export
+modelName.ets <- function(object, ...){
+    return(object$method);
+}
+
+#' @export
+modelName.forecast <- function(object, ...){
+    return(object$method);
+}
+
+#' @export
+modelName.smooth <- function(object, ...){
+    return(object$model);
+}
+
 #### Function extracts type of model. For example "AAN" from ets ####
 #' @export
 modelType.default <- function(object, ...){
@@ -1031,11 +1016,12 @@ modelType.default <- function(object, ...){
 #' @export
 modelType.smooth <- function(object, ...){
     model <- object$model;
+    smoothType <- smoothType(object);
 
-    if(gregexpr("ETS",model)!=-1){
+    if(smoothType=="ETS"){
         modelType <- substring(model,unlist(gregexpr("\\(",model))+1,unlist(gregexpr("\\)",model))-1);
     }
-    else if(gregexpr("CES",model)!=-1){
+    else if(smoothType=="CES"){
         modelType <- substring(model,unlist(gregexpr("\\(",model))+1,unlist(gregexpr("\\)",model))-1);
         if(modelType=="n"){
             modelType <- "none";
@@ -1068,18 +1054,18 @@ modelType.iss <- function(object, ...){
 #### Function extracts orders of provided model ####
 #' @export
 orders.default <- function(object, ...){
-    orders <- NA;
-    return(orders);
+    return(NA);
 }
 
 #' @export
 orders.smooth <- function(object, ...){
+    smoothType <- smoothType(object);
     model <- object$model;
     if(!is.null(model)){
-        if(gregexpr("GES",model)!=-1){
+        if(smoothType=="GUM"){
             orders <- as.numeric(substring(model,unlist(gregexpr("\\[",model))-1,unlist(gregexpr("\\[",model))-1));
         }
-        else if(gregexpr("ARIMA",model)!=-1){
+        else if(smoothType=="ARIMA"){
             if(any(unlist(gregexpr("combine",object$model))==-1)){
                 arima.orders <- paste0(c("",substring(model,unlist(gregexpr("\\(",model))+1,unlist(gregexpr("\\)",model))-1),"")
                                        ,collapse=";");
@@ -1095,10 +1081,10 @@ orders.smooth <- function(object, ...){
                 warning("ARIMA was combined and we cannot extract orders anymore. Sorry!",call.=FALSE);
             }
         }
-        else if(gregexpr("SMA",model)!=-1){
+        else if(smoothType=="SMA"){
             orders <- as.numeric(substring(model,unlist(gregexpr("\\(",model))+1,unlist(gregexpr("\\)",model))-1));
         }
-        else if(gregexpr("ETS",model)!=-1){
+        else if(smoothType=="ETS"){
             modelName <- modelType(object);
             orders <- 1;
             if(substr(modelName,2,2)!="N"){
@@ -1108,7 +1094,7 @@ orders.smooth <- function(object, ...){
                 orders <- c(orders, 1);
             }
         }
-        else if(gregexpr("CES",model)!=-1){
+        else if(smoothType=="CES"){
             modelName <- modelType(object);
             if(modelName=="none"){
                 orders <- 2;
@@ -1141,6 +1127,11 @@ orders.smooth <- function(object, ...){
 #' @export
 orders.smooth.sim <- orders.smooth;
 
+#' @export
+orders.ar <- function(object, ...){
+    return(list(ar=object$order));
+}
+
 #' @export
 orders.Arima <- function(object, ...){
     model <- object$arma;
@@ -1160,7 +1151,8 @@ orders.Arima <- function(object, ...){
 plot.smooth <- function(x, ...){
     ellipsis <- list(...);
     parDefault <- par(no.readonly = TRUE);
-    if(gregexpr("ETS",x$model)!=-1){
+    smoothType <- smoothType(x);
+    if(smoothType=="ETS"){
         if(any(unlist(gregexpr("C",x$model))==-1)){
             if(ncol(x$states)>10){
                 message("Too many states. Plotting them one by one on several graphs.");
@@ -1192,6 +1184,17 @@ plot.smooth <- function(x, ...){
             message("Combination of models was done. Sorry, but there is nothing to plot.");
         }
     }
+    else if(smoothType=="CMA"){
+        ellipsis$actuals <- x$actuals;
+        ellipsis$forecast <- x$forecast;
+        ellipsis$fitted <- x$fitted;
+        ellipsis$legend <- FALSE;
+        ellipsis$vline <- FALSE;
+        if(is.null(ellipsis$main)){
+            ellipsis$main <- x$model;
+        }
+        do.call(graphmaker, ellipsis);
+    }
     else{
         if(any(unlist(gregexpr("combine",x$model))!=-1)){
             # If we did combinations, we cannot do anything
@@ -1307,9 +1310,25 @@ plot.iss <- function(x, ...){
 #### Prints of smooth ####
 #' @export
 print.smooth <- function(x, ...){
-    holdout <- any(!is.na(x$holdout));
-    intervals <- any(!is.na(x$lower));
-    cumulative <- x$cumulative;
+    smoothType <- smoothType(x);
+
+    if(!is.list(x$model)){
+        if(smoothType=="CMA"){
+            holdout <- FALSE;
+            intervals <- FALSE;
+            cumulative <- FALSE;
+        }
+        else{
+            holdout <- any(!is.na(x$holdout));
+            intervals <- any(!is.na(x$lower));
+            cumulative <- x$cumulative;
+        }
+    }
+    else{
+        holdout <- any(!is.na(x$holdout));
+        intervals <- any(!is.na(x$lower));
+        cumulative <- x$cumulative;
+    }
 
     if(all(holdout,intervals)){
         if(!cumulative){
@@ -1327,12 +1346,12 @@ print.smooth <- function(x, ...){
 
     if(!is.null(x$model)){
         if(!is.list(x$model)){
-            if(gregexpr("SMA",x$model)!=-1){
+            if(any(smoothType==c("SMA","CMA"))){
                 x$iprob <- 1;
                 x$initialType <- "b";
                 intermittent <- "n";
             }
-            else if(gregexpr("ETS",x$model)!=-1){
+            else if(smoothType=="ETS"){
                 # If cumulative forecast and Etype=="M", report that this was "parameteric" interval
                 if(cumulative & substr(modelType(x),1,1)=="M"){
                     intervalsType <- "p";
@@ -1358,6 +1377,7 @@ print.smooth <- function(x, ...){
 
 #' @export
 print.smooth.sim <- function(x, ...){
+    smoothType <- smoothType(x);
     if(is.null(dim(x$data))){
         nsim <- 1
     }
@@ -1369,7 +1389,7 @@ print.smooth.sim <- function(x, ...){
     cat(paste0("Number of generated series: ",nsim,"\n"));
 
     if(nsim==1){
-        if(gregexpr("ETS",x$model)!=-1){
+        if(smoothType=="ETS"){
             cat(paste0("Persistence vector: \n"));
             xPersistence <- as.vector(x$persistence);
             names(xPersistence) <- rownames(x$persistence);
@@ -1382,7 +1402,7 @@ print.smooth.sim <- function(x, ...){
             }
             cat(paste0("True likelihood: ",round(x$logLik,3),"\n"));
         }
-        else if(gregexpr("ARIMA",x$model)!=-1){
+        else if(smoothType=="ARIMA"){
             ar.orders <- orders(x)$ar;
             i.orders <- orders(x)$i;
             ma.orders <- orders(x)$ma;
@@ -1443,7 +1463,7 @@ print.smooth.sim <- function(x, ...){
             }
             cat(paste0("True likelihood: ",round(x$logLik,3),"\n"));
         }
-        else if(gregexpr("CES",x$model)!=-1){
+        else if(smoothType=="CES"){
             cat(paste0("Smoothing parameter A: ",round(x$A,3),"\n"));
             if(!is.null(x$B)){
                 if(is.complex(x$B)){
@@ -1455,7 +1475,7 @@ print.smooth.sim <- function(x, ...){
             }
             cat(paste0("True likelihood: ",round(x$logLik,3),"\n"));
         }
-        else if(gregexpr("SMA",x$model)!=-1){
+        else if(smoothType=="SMA"){
             cat(paste0("True likelihood: ",round(x$logLik,3),"\n"));
         }
     }
@@ -1521,6 +1541,7 @@ print.iss <- function(x, ...){
 #' @export
 simulate.smooth <- function(object, nsim=1, seed=NULL, obs=NULL, ...){
     ellipsis <- list(...);
+    smoothType <- smoothType(object);
     if(is.null(obs)){
         obs <- nobs(object);
     }
@@ -1592,7 +1613,7 @@ simulate.smooth <- function(object, nsim=1, seed=NULL, obs=NULL, ...){
     args$initial <- object$initial;
     args$iprob <- object$iprob[length(object$iprob)];
 
-    if(gregexpr("ETS",object$model)!=-1){
+    if(smoothType=="ETS"){
         model <- modelType(object);
         if(any(unlist(gregexpr("C",model))==-1)){
             args <- c(args,list(model=model, phi=object$phi, persistence=object$persistence,
@@ -1605,7 +1626,7 @@ simulate.smooth <- function(object, nsim=1, seed=NULL, obs=NULL, ...){
             simulatedData <- NA;
         }
     }
-    else if(gregexpr("ARIMA",object$model)!=-1){
+    else if(smoothType=="ARIMA"){
         if(any(unlist(gregexpr("combine",object$model))==-1)){
             args <- c(args,list(orders=orders(object), lags=lags(object),
                                 AR=object$AR, MA=object$MA, constant=object$constant));
@@ -1617,19 +1638,19 @@ simulate.smooth <- function(object, nsim=1, seed=NULL, obs=NULL, ...){
             simulatedData <- NA;
         }
     }
-    else if(gregexpr("CES",object$model)!=-1){
+    else if(smoothType=="CES"){
         args <- c(args,list(seasonality=modelType(object), A=object$A, B=object$B));
 
         simulatedData <- do.call("sim.ces",args);
     }
-    else if(gregexpr("GES",object$model)!=-1){
+    else if(smoothType=="GUM"){
         args <- c(args,list(orders=orders(object), lags=lags(object),
                             measurement=object$measurement, transition=object$transition,
                             persistence=object$persistence));
 
-        simulatedData <- do.call("sim.ges",args);
+        simulatedData <- do.call("sim.gum",args);
     }
-    else if(gregexpr("SMA",object$model)!=-1){
+    else if(smoothType=="SMA"){
         args <- c(args,list(order=orders(object)));
 
         simulatedData <- do.call("sim.sma",args);
@@ -1642,6 +1663,47 @@ simulate.smooth <- function(object, nsim=1, seed=NULL, obs=NULL, ...){
     return(simulatedData);
 }
 
+#### Type of smooth model. Internal function ####
+smoothType.default <- function(object, ...){
+    return(NA);
+}
+
+smoothType.smooth <- function(object, ...){
+    if(!is.list(object$model)){
+        if(gregexpr("ETS",object$model)!=-1){
+            smoothType <- "ETS";
+        }
+        else if(gregexpr("CES",object$model)!=-1){
+            smoothType <- "CES";
+        }
+        else if(gregexpr("ARIMA",object$model)!=-1){
+            smoothType <- "ARIMA";
+        }
+        else if(gregexpr("GUM",object$model)!=-1){
+            smoothType <- "GUM";
+        }
+        else if(gregexpr("SMA",object$model)!=-1){
+            smoothType <- "SMA";
+        }
+        else if(gregexpr("CMA",object$model)!=-1){
+            smoothType <- "CMA";
+        }
+        else if(gregexpr("VES",object$model)!=-1){
+            smoothType <- "VES";
+        }
+        else{
+            smoothType <- NA;
+        }
+    }
+    else{
+        smoothType <- "smoothCombine";
+    }
+
+    return(smoothType);
+}
+
+smoothType.smooth.sim <- smoothType.smooth;
+
 #### Summary of objects ####
 #' @export
 summary.smooth <- function(object, ...){
