@@ -1,6 +1,6 @@
 utils::globalVariables(c("normalizer","constantValue","constantRequired","constantEstimate","C",
                          "ARValue","ARRequired","AREstimate","MAValue","MARequired","MAEstimate",
-                         "yForecastStart"));
+                         "yForecastStart","nonZeroARI","nonZeroMA"));
 
 #' State Space ARIMA
 #'
@@ -60,6 +60,8 @@ utils::globalVariables(c("normalizer","constantValue","constantRequired","consta
 #' \code{i} or \code{ma} in \code{orders} variable. There is no restrictions on
 #' the length of \code{lags} vector. It is recommended to order \code{lags}
 #' ascending.
+#' The orders are set by a user. If you want the automatic order selection,
+#' then use \link[smooth]{auto.ssarima} function instead.
 #' @param constant If \code{TRUE}, constant term is included in the model. Can
 #' also be a number (constant value).
 #' @param AR Vector or matrix of AR parameters. The order of parameters should
@@ -72,9 +74,6 @@ utils::globalVariables(c("normalizer","constantValue","constantRequired","consta
 #' passed here.
 #' @param ...  Other non-documented parameters.
 #'
-#' Vectors of orders can be passed here using \code{ar.orders}, \code{i.orders}
-#' and \code{ma.orders}. \code{orders} variable needs to be NULL in this case.
-#'
 #' Parameter \code{model} can accept a previously estimated SSARIMA model and
 #' use all its parameters.
 #'
@@ -103,8 +102,8 @@ utils::globalVariables(c("normalizer","constantValue","constantRequired","consta
 #' \item \code{nParam} - table with the number of estimated / provided parameters.
 #' If a previous model was reused, then its initials are reused and the number of
 #' provided parameters will take this into account.
-#' \item \code{fitted} - the fitted values of ETS.
-#' \item \code{forecast} - the point forecast of ETS.
+#' \item \code{fitted} - the fitted values.
+#' \item \code{forecast} - the point forecast.
 #' \item \code{lower} - the lower bound of prediction interval. When
 #' \code{intervals="none"} then NA is returned.
 #' \item \code{upper} - the higher bound of prediction interval. When
@@ -144,8 +143,8 @@ utils::globalVariables(c("normalizer","constantValue","constantRequired","consta
 #' \code{holdout=TRUE}.
 #' }
 #'
-#' @seealso \code{\link[forecast]{auto.arima}, \link[smooth]{orders},
-#' \link[smooth]{lags}, \link[smooth]{sim.ssarima}}
+#' @seealso \code{\link[smooth]{auto.ssarima}, \link[smooth]{orders},
+#' \link[smooth]{lags}, \link[smooth]{sim.ssarima}, \link[forecast]{auto.arima}}
 #'
 #' @examples
 #'
@@ -172,7 +171,7 @@ utils::globalVariables(c("normalizer","constantValue","constantRequired","consta
 #' ssarima(rnorm(118,100,3),orders=list(ar=c(1),i=c(1),ma=c(1,1)),
 #'         lags=c(1,4),h=18,holdout=TRUE,initial="o")}
 #'
-#' # SARIMA of a perculiar order on AirPassengers data
+#' # SARIMA of a peculiar order on AirPassengers data
 #' \dontrun{ssarima(AirPassengers,orders=list(ar=c(1,0,3),i=c(1,0,1),ma=c(0,1,2)),lags=c(1,6,12),
 #'         h=10,holdout=TRUE)}
 #'
@@ -220,7 +219,7 @@ ssarima <- function(data, orders=list(ar=c(0),i=c(1),ma=c(1)), lags=c(1),
         if(is.null(model$model)){
             stop("The provided model is not ARIMA.",call.=FALSE);
         }
-        else if(gregexpr("ARIMA",model$model)==-1){
+        else if(smoothType(model)!="ARIMA"){
             stop("The provided model is not ARIMA.",call.=FALSE);
         }
 
@@ -255,13 +254,13 @@ ssarima <- function(data, orders=list(ar=c(0),i=c(1),ma=c(1)), lags=c(1),
             MA <- model$MA;
             constant <- model$constant;
             model <- model$model;
-            arima.orders <- paste0(c("",substring(model,unlist(gregexpr("\\(",model))+1,unlist(gregexpr("\\)",model))-1),"")
+            arimaOrders <- paste0(c("",substring(model,unlist(gregexpr("\\(",model))+1,unlist(gregexpr("\\)",model))-1),"")
                                    ,collapse=";");
-            comas <- unlist(gregexpr("\\,",arima.orders));
-            semicolons <- unlist(gregexpr("\\;",arima.orders));
-            ar.orders <- as.numeric(substring(arima.orders,semicolons[-length(semicolons)]+1,comas[2*(1:(length(comas)/2))-1]-1));
-            i.orders <- as.numeric(substring(arima.orders,comas[2*(1:(length(comas)/2))-1]+1,comas[2*(1:(length(comas)/2))-1]+1));
-            ma.orders <- as.numeric(substring(arima.orders,comas[2*(1:(length(comas)/2))]+1,semicolons[-1]-1));
+            comas <- unlist(gregexpr("\\,",arimaOrders));
+            semicolons <- unlist(gregexpr("\\;",arimaOrders));
+            ar.orders <- as.numeric(substring(arimaOrders,semicolons[-length(semicolons)]+1,comas[2*(1:(length(comas)/2))-1]-1));
+            i.orders <- as.numeric(substring(arimaOrders,comas[2*(1:(length(comas)/2))-1]+1,comas[2*(1:(length(comas)/2))-1]+1));
+            ma.orders <- as.numeric(substring(arimaOrders,comas[2*(1:(length(comas)/2))]+1,semicolons[-1]-1));
             if(any(unlist(gregexpr("\\[",model))!=-1)){
                 lags <- as.numeric(substring(model,unlist(gregexpr("\\[",model))+1,unlist(gregexpr("\\]",model))-1));
             }
@@ -328,40 +327,9 @@ CF <- function(C){
                            nExovars, matxt, matat, matFX, vecgX, ot,
                            AREstimate, MAEstimate, constantRequired, constantEstimate,
                            xregEstimate, updateX, FXEstimate, gXEstimate, initialXEstimate,
-                           bounds);
-
-    # elements <- polysoswrap(ar.orders, ma.orders, i.orders, lags, nComponents,
-    #                         ARValue, MAValue, constantValue, C,
-    #                         matvt, vecg, matF,
-    #                         initialType, nExovars, matat, matFX, vecgX,
-    #                         AREstimate, MAEstimate, constantRequired, constantEstimate,
-    #                         xregEstimate, updateX, FXEstimate, gXEstimate, initialXEstimate);
-    # matF <- elements$matF;
-    # vecg <- elements$vecg;
-    # matvt[,] <- elements$matvt;
-    # matvt[1,] <- matrixPowerWrap(matF,nComponents+1) %*% matvt[1,];
-    # matat[,] <- elements$matat;
-    # matFX <- elements$matFX;
-    # vecgX <- elements$vecgX;
-
-    # polysos.ar <- elements$arPolynomial;
-    # polysos.ma <- elements$maPolynomial;
-    #
-    # if(bounds=="a" & (nComponents > 0)){
-    #     arRoots <- abs(polyroot(polysos.ar));
-    #     if(any(arRoots<1)){
-    #         return(max(arRoots)*1E+100);
-    #     }
-    #     maRoots <- abs(polyroot(polysos.ma));
-    #     if(any(maRoots<1)){
-    #         return(max(maRoots)*1E+100);
-    #     }
-    # }
-    #
-    # cfRes <- optimizerwrap(elements$matvt, elements$matF, matw, y, elements$vecg,
-    #                        h, modellags, Etype, Ttype, Stype,
-    #                        multisteps, cfType, normalizer, initialType,
-    #                        matxt, elements$matat, elements$matFX, elements$vecgX, ot);
+                           bounds,
+                           # The last bit is "ssarimaOld"
+                           TRUE, nonZeroARI, nonZeroMA);
 
     if(is.nan(cfRes) | is.na(cfRes) | is.infinite(cfRes)){
         cfRes <- 1e+100;
@@ -485,11 +453,11 @@ CreatorSSARIMA <- function(silentText=FALSE,...){
             matvt[1,1:nComponents] <- initialValue;
         }
         else{
-            if(obsInsample<(nComponents+datafreq)){
+            if(obsInsample<(nComponents+dataFreq)){
                 matvt[1:nComponents,] <- y[1:nComponents] + diff(y[1:(nComponents+1)]);
             }
             else{
-                matvt[1:nComponents,] <- (y[1:nComponents]+y[1:nComponents+datafreq])/2;
+                matvt[1:nComponents,] <- (y[1:nComponents]+y[1:nComponents+dataFreq])/2;
             }
         }
     }
@@ -500,9 +468,9 @@ CreatorSSARIMA <- function(silentText=FALSE,...){
         modellags <- matrix(1,1,1);
     }
 
-##### Preset y.fit, y.for, errors and basic parameters #####
-    y.fit <- rep(NA,obsInsample);
-    y.for <- rep(NA,h);
+##### Preset yFitted, yForecast, errors and basic parameters #####
+    yFitted <- rep(NA,obsInsample);
+    yForecast <- rep(NA,h);
     errors <- rep(NA,obsInsample);
 
 ##### Prepare exogenous variables #####
@@ -559,7 +527,7 @@ CreatorSSARIMA <- function(silentText=FALSE,...){
         # If transition is provided and not identity, and other things are provided, write them as "provided"
         parametersNumber[2,2] <- (length(matFX)*(!is.null(transitionX) & !all(matFX==diag(ncol(matat)))) +
                                       nrow(vecgX)*(!is.null(persistenceX)) +
-                                      ncol(matat)*(!is.null(initialX)) - nParamExo);
+                                      ncol(matat)*(!is.null(initialX)));
     }
 
 ##### Check number of observations vs number of max parameters #####
@@ -679,7 +647,9 @@ CreatorSSARIMA <- function(silentText=FALSE,...){
                                 matvt, vecg, matF,
                                 initialType, nExovars, matat, matFX, vecgX,
                                 AREstimate, MAEstimate, constantRequired, constantEstimate,
-                                xregEstimate, updateX, FXEstimate, gXEstimate, initialXEstimate);
+                                xregEstimate, updateX, FXEstimate, gXEstimate, initialXEstimate,
+                                # The last bit is "ssarimaOld"
+                                TRUE, modellags, nonZeroARI, nonZeroMA);
         matF <- elements$matF;
         vecg <- elements$vecg;
         matvt[,] <- elements$matvt;
@@ -688,9 +658,6 @@ CreatorSSARIMA <- function(silentText=FALSE,...){
         vecgX <- elements$vecgX;
         polysos.ar <- elements$arPolynomial;
         polysos.ma <- elements$maPolynomial;
-        # Need to remove polyroot() here as well and find a better substitution
-        arRoots <- abs(polyroot(polysos.ar));
-        maRoots <- abs(polyroot(polysos.ma));
 
         ssFitter(ParentEnvironment=environment());
 
@@ -746,7 +713,9 @@ CreatorSSARIMA <- function(silentText=FALSE,...){
                             matvt, vecg, matF,
                             initialType, nExovars, matat, matFX, vecgX,
                             AREstimate, MAEstimate, constantRequired, constantEstimate,
-                            xregEstimate, updateX, FXEstimate, gXEstimate, initialXEstimate);
+                            xregEstimate, updateX, FXEstimate, gXEstimate, initialXEstimate,
+                            # The last bit is "ssarimaOld"
+                            TRUE, modellags, nonZeroARI, nonZeroMA);
     matF <- elements$matF;
     vecg <- elements$vecg;
     matvt[,] <- elements$matvt;
@@ -755,16 +724,8 @@ CreatorSSARIMA <- function(silentText=FALSE,...){
     vecgX <- elements$vecgX;
     polysos.ar <- elements$arPolynomial;
     polysos.ma <- elements$maPolynomial;
-    # Need to remove polyroot() here as well and find a better substitution
-    arRoots <- polyroot(polysos.ar);
-    maRoots <- polyroot(polysos.ma);
 
     nComponents <- nComponents + constantRequired;
-    # Write down Fisher Information if needed
-    if(FI){
-        environment(likelihoodFunction) <- environment();
-        FI <- -numDeriv::hessian(likelihoodFunction,C);
-    }
 
 ##### Fit simple model and produce forecast #####
     ssFitter(ParentEnvironment=environment());
@@ -774,7 +735,7 @@ CreatorSSARIMA <- function(silentText=FALSE,...){
 
 # Write down initials of states vector and exogenous
     if(initialType!="p"){
-        if(constantRequired==TRUE){
+        if(constantRequired){
             initialValue <- matvt[1,-ncol(matvt)];
         }
         else{
@@ -809,7 +770,7 @@ CreatorSSARIMA <- function(silentText=FALSE,...){
     parametersNumber[1,1] <- parametersNumber[1,1] + 1;
 
     # Write down the probabilities from intermittent models
-    pt <- ts(c(as.vector(pt),as.vector(pt.for)),start=dataStart,frequency=datafreq);
+    pt <- ts(c(as.vector(pt),as.vector(pForecast)),start=dataStart,frequency=dataFreq);
     # Write down the number of parameters of imodel
     if(all(intermittent!=c("n","provided")) & !imodelProvided){
         parametersNumber[1,3] <- imodel$nParam;
@@ -876,53 +837,37 @@ CreatorSSARIMA <- function(silentText=FALSE,...){
         MAterms <- NULL;
     }
 
-    n.coef <- ar.coef <- ma.coef <- 0;
-    ar.i <- ma.i <- 1;
+    nCoef <- arCoef <- maCoef <- 0;
+    arIndex <- maIndex <- 1;
     for(i in 1:length(ar.orders)){
         if(ar.orders[i]!=0){
             if(AREstimate){
-                ARterms[1:ar.orders[i],ar.i] <- C[n.coef+(1:ar.orders[i])];
-                n.coef <- n.coef + ar.orders[i];
+                ARterms[1:ar.orders[i],arIndex] <- C[nCoef+(1:ar.orders[i])];
+                names(C)[nCoef+(1:ar.orders[i])] <- paste0("AR(",1:ar.orders[i],"), ",colnames(ARterms)[arIndex]);
+                nCoef <- nCoef + ar.orders[i];
                 parametersNumber[1,1] <- parametersNumber[1,1] + ar.orders[i];
             }
             else{
-                ARterms[1:ar.orders[i],ar.i] <- ARValue[ar.coef+(1:ar.orders[i])];
-                ar.coef <- ar.coef + ar.orders[i];
+                ARterms[1:ar.orders[i],arIndex] <- ARValue[arCoef+(1:ar.orders[i])];
+                arCoef <- arCoef + ar.orders[i];
             }
-            ar.i <- ar.i + 1;
+            arIndex <- arIndex + 1;
         }
         if(ma.orders[i]!=0){
             if(MAEstimate){
-                MAterms[1:ma.orders[i],ma.i] <- C[n.coef+(1:ma.orders[i])];
-                n.coef <- n.coef + ma.orders[i];
+                MAterms[1:ma.orders[i],maIndex] <- C[nCoef+(1:ma.orders[i])];
+                names(C)[nCoef+(1:ma.orders[i])] <- paste0("MA(",1:ma.orders[i],"), ",colnames(MAterms)[maIndex]);
+                nCoef <- nCoef + ma.orders[i];
                 parametersNumber[1,1] <- parametersNumber[1,1] + ma.orders[i];
             }
             else{
-                MAterms[1:ma.orders[i],ma.i] <- MAValue[ma.coef+(1:ma.orders[i])];
-                ma.coef <- ma.coef + ma.orders[i];
+                MAterms[1:ma.orders[i],maIndex] <- MAValue[maCoef+(1:ma.orders[i])];
+                maCoef <- maCoef + ma.orders[i];
             }
-            ma.i <- ma.i + 1;
+            maIndex <- maIndex + 1;
         }
     }
 
-    if(holdout==T){
-        y.holdout <- ts(data[(obsInsample+1):obsAll],start=yForecastStart,frequency=frequency(data));
-        if(cumulative){
-            errormeasures <- Accuracy(sum(y.holdout),y.for,h*y);
-        }
-        else{
-            errormeasures <- Accuracy(y.holdout,y.for,y);
-        }
-
-        if(cumulative){
-            y.holdout <- ts(sum(y.holdout),start=yForecastStart,frequency=datafreq);
-        }
-    }
-    else{
-        y.holdout <- NA;
-        errormeasures <- NA;
-    }
-
 # Give model the name
     if((length(ar.orders)==1) && all(lags==1)){
         if(!is.null(xreg)){
@@ -955,6 +900,12 @@ CreatorSSARIMA <- function(silentText=FALSE,...){
         if(constantEstimate){
             constantValue <- matvt[1,nComponents];
             parametersNumber[1,1] <- parametersNumber[1,1] + 1;
+            if(!is.null(names(C))){
+                names(C)[is.na(names(C))][1] <- "Constant";
+            }
+            else{
+                names(C)[1] <- "Constant";
+            }
         }
         const <- constantValue;
 
@@ -969,29 +920,66 @@ CreatorSSARIMA <- function(silentText=FALSE,...){
         const <- FALSE;
         constantValue <- NULL;
     }
+#
+#     if(initialType=="o"){
+#         names(C)[is.na(names(C))] <- paste0("Component ",c(1:length(initialValue)));
+#     }
 
     parametersNumber[1,4] <- sum(parametersNumber[1,1:3]);
     parametersNumber[2,4] <- sum(parametersNumber[2,1:3]);
 
+    # Write down Fisher Information if needed
+    if(FI & parametersNumber[1,4]>1){
+        environment(likelihoodFunction) <- environment();
+        FI <- -numDeriv::hessian(likelihoodFunction,C);
+        rownames(FI) <- colnames(FI) <- names(C);
+        if(initialType=="o"){
+            # Leave only AR and MA parameters. Forget about the initials and exogenous
+            FI <- FI[!is.na(rownames(FI)),!is.na(colnames(FI))];
+        }
+    }
+    else{
+        FI <- NA;
+    }
+
+##### Deal with the holdout sample #####
+    if(holdout){
+        yHoldout <- ts(data[(obsInsample+1):obsAll],start=yForecastStart,frequency=frequency(data));
+        if(cumulative){
+            errormeasures <- Accuracy(sum(yHoldout),yForecast,h*y);
+        }
+        else{
+            errormeasures <- Accuracy(yHoldout,yForecast,y);
+        }
+
+        if(cumulative){
+            yHoldout <- ts(sum(yHoldout),start=yForecastStart,frequency=dataFreq);
+        }
+    }
+    else{
+        yHoldout <- NA;
+        errormeasures <- NA;
+    }
+
 ##### Make a plot #####
     if(!silentGraph){
-        y.for.new <- y.for;
-        y.high.new <- y.high;
-        y.low.new <- y.low;
+        yForecastNew <- yForecast;
+        yUpperNew <- yUpper;
+        yLowerNew <- yLower;
         if(cumulative){
-            y.for.new <- ts(rep(y.for/h,h),start=yForecastStart,frequency=datafreq)
+            yForecastNew <- ts(rep(yForecast/h,h),start=yForecastStart,frequency=dataFreq)
             if(intervals){
-                y.high.new <- ts(rep(y.high/h,h),start=yForecastStart,frequency=datafreq)
-                y.low.new <- ts(rep(y.low/h,h),start=yForecastStart,frequency=datafreq)
+                yUpperNew <- ts(rep(yUpper/h,h),start=yForecastStart,frequency=dataFreq)
+                yLowerNew <- ts(rep(yLower/h,h),start=yForecastStart,frequency=dataFreq)
             }
         }
 
         if(intervals){
-            graphmaker(actuals=data,forecast=y.for.new,fitted=y.fit, lower=y.low.new,upper=y.high.new,
+            graphmaker(actuals=data,forecast=yForecastNew,fitted=yFitted, lower=yLowerNew,upper=yUpperNew,
                        level=level,legend=!silentLegend,main=modelname,cumulative=cumulative);
         }
         else{
-            graphmaker(actuals=data,forecast=y.for.new,fitted=y.fit,
+            graphmaker(actuals=data,forecast=yForecastNew,fitted=yFitted,
                        legend=!silentLegend,main=modelname,cumulative=cumulative);
         }
     }
@@ -1003,9 +991,9 @@ CreatorSSARIMA <- function(silentText=FALSE,...){
                   AR=ARterms,I=Iterms,MA=MAterms,constant=const,
                   initialType=initialType,initial=initialValue,
                   nParam=parametersNumber,
-                  fitted=y.fit,forecast=y.for,lower=y.low,upper=y.high,residuals=errors,
+                  fitted=yFitted,forecast=yForecast,lower=yLower,upper=yUpper,residuals=errors,
                   errors=errors.mat,s2=s2,intervals=intervalsType,level=level,cumulative=cumulative,
-                  actuals=data,holdout=y.holdout,imodel=imodel,
+                  actuals=data,holdout=yHoldout,imodel=imodel,
                   xreg=xreg,updateX=updateX,initialX=initialX,persistenceX=persistenceX,transitionX=transitionX,
                   ICs=ICs,logLik=logLik,cf=cfObjective,cfType=cfType,FI=FI,accuracy=errormeasures);
     return(structure(model,class="smooth"));
