@@ -22,7 +22,7 @@
 #' @param probability Type of probability assumed in the model. If
 #' \code{"dependent"}, then it is assumed that occurrence of one variable is
 #' connected with the occurrence with another one. In case of \code{"independent"}
-#' the occurrence of the variables is assumed to happen independ of each
+#' the occurrence of the variables is assumed to happen independent of each
 #' other.
 #' @param model Type of ETS model used for the estimation. Normally this should
 #' be either \code{"ANN"} or \code{"MNN"}. If you assume that there are some
@@ -76,7 +76,7 @@
 #'
 #' @export viss
 viss <- function(data, intermittent=c("logistic","none","fixed"),
-                 ic=c("AICc","AIC","BIC"), h=10, holdout=FALSE,
+                 ic=c("AICc","AIC","BIC","BICc"), h=10, holdout=FALSE,
                  probability=c("dependent","independent"),
                  model="ANN", persistence=NULL, transition=NULL, phi=NULL,
                  initial=NULL, initialSeason=NULL, xreg=NULL){
